"""
Python module generated from Java source file java.util.Date

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from java.lang.ref import SoftReference
from java.text import DateFormat
from java.time import Instant
from java.time import LocalDate
from java.util import *
from sun.util.calendar import BaseCalendar
from sun.util.calendar import CalendarDate
from sun.util.calendar import CalendarSystem
from sun.util.calendar import CalendarUtils
from sun.util.calendar import Era
from sun.util.calendar import Gregorian
from sun.util.calendar import ZoneInfo
from typing import Any, Callable, Iterable, Tuple


class Date(Serializable, Cloneable, Comparable):
    """
    The class `Date` represents a specific instant
    in time, with millisecond precision.
    
    Prior to JDK&nbsp;1.1, the class `Date` had two additional
    functions.  It allowed the interpretation of dates as year, month, day, hour,
    minute, and second values.  It also allowed the formatting and parsing
    of date strings.  Unfortunately, the API for these functions was not
    amenable to internationalization.  As of JDK&nbsp;1.1, the
    `Calendar` class should be used to convert between dates and time
    fields and the `DateFormat` class should be used to format and
    parse date strings.
    The corresponding methods in `Date` are deprecated.
    
    Although the `Date` class is intended to reflect
    coordinated universal time (UTC), it may not do so exactly,
    depending on the host environment of the Java Virtual Machine.
    Nearly all modern operating systems assume that 1&nbsp;day&nbsp;=
    24&nbsp;&times;&nbsp;60&nbsp;&times;&nbsp;60&nbsp;= 86400 seconds
    in all cases. In UTC, however, about once every year or two there
    is an extra second, called a "leap second." The leap
    second is always added as the last second of the day, and always
    on December 31 or June 30. For example, the last minute of the
    year 1995 was 61 seconds long, thanks to an added leap second.
    Most computer clocks are not accurate enough to be able to reflect
    the leap-second distinction.
    
    Some computer standards are defined in terms of Greenwich mean
    time (GMT), which is equivalent to universal time (UT).  GMT is
    the "civil" name for the standard; UT is the
    "scientific" name for the same standard. The
    distinction between UTC and UT is that UTC is based on an atomic
    clock and UT is based on astronomical observations, which for all
    practical purposes is an invisibly fine hair to split. Because the
    earth's rotation is not uniform (it slows down and speeds up
    in complicated ways), UT does not always flow uniformly. Leap
    seconds are introduced as needed into UTC so as to keep UTC within
    0.9 seconds of UT1, which is a version of UT with certain
    corrections applied. There are other time and date systems as
    well; for example, the time scale used by the satellite-based
    global positioning system (GPS) is synchronized to UTC but is
    *not* adjusted for leap seconds. An interesting source of
    further information is the United States Naval Observatory (USNO):
    <blockquote>```
        <a href="https://www.usno.navy.mil/USNO">https://www.usno.navy.mil/USNO</a>
    ```</blockquote>
    
    and the material regarding "Systems of Time" at:
    <blockquote>```
        <a href="https://www.usno.navy.mil/USNO/time/master-clock/systems-of-time">https://www.usno.navy.mil/USNO/time/master-clock/systems-of-time</a>
    ```</blockquote>
    
    which has descriptions of various different time systems including
    UT, UT1, and UTC.
    
    In all methods of class `Date` that accept or return
    year, month, date, hours, minutes, and seconds values, the
    following representations are used:
    
    - A year *y* is represented by the integer
        *y*&nbsp;`- 1900`.
    - A month is represented by an integer from 0 to 11; 0 is January,
        1 is February, and so forth; thus 11 is December.
    - A date (day of month) is represented by an integer from 1 to 31
        in the usual manner.
    - An hour is represented by an integer from 0 to 23. Thus, the hour
        from midnight to 1 a.m. is hour 0, and the hour from noon to 1
        p.m. is hour 12.
    - A minute is represented by an integer from 0 to 59 in the usual manner.
    - A second is represented by an integer from 0 to 61; the values 60 and
        61 occur only for leap seconds and even then only in Java
        implementations that actually track leap seconds correctly. Because
        of the manner in which leap seconds are currently introduced, it is
        extremely unlikely that two leap seconds will occur in the same
        minute, but this specification follows the date and time conventions
        for ISO C.
    
    
    In all cases, arguments given to methods for these purposes need
    not fall within the indicated ranges; for example, a date may be
    specified as January 32 and is interpreted as meaning February 1.

    Author(s)
    - Alan Liu

    See
    - java.util.TimeZone

    Since
    - 1.0
    """

    def __init__(self):
        """
        Allocates a `Date` object and initializes it so that
        it represents the time at which it was allocated, measured to the
        nearest millisecond.

        See
        - java.lang.System.currentTimeMillis()
        """
        ...


    def __init__(self, date: int):
        """
        Allocates a `Date` object and initializes it to
        represent the specified number of milliseconds since the
        standard base time known as "the epoch", namely January 1,
        1970, 00:00:00 GMT.

        Arguments
        - date: the milliseconds since January 1, 1970, 00:00:00 GMT.

        See
        - java.lang.System.currentTimeMillis()
        """
        ...


    def __init__(self, year: int, month: int, date: int):
        """
        Allocates a `Date` object and initializes it so that
        it represents midnight, local time, at the beginning of the day
        specified by the `year`, `month`, and
        `date` arguments.

        Arguments
        - year: the year minus 1900.
        - month: the month between 0-11.
        - date: the day of the month between 1-31.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.set(year + 1900, month, date)`
        or `GregorianCalendar(year + 1900, month, date)`.
        """
        ...


    def __init__(self, year: int, month: int, date: int, hrs: int, min: int):
        """
        Allocates a `Date` object and initializes it so that
        it represents the instant at the start of the minute specified by
        the `year`, `month`, `date`,
        `hrs`, and `min` arguments, in the local
        time zone.

        Arguments
        - year: the year minus 1900.
        - month: the month between 0-11.
        - date: the day of the month between 1-31.
        - hrs: the hours between 0-23.
        - min: the minutes between 0-59.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.set(year + 1900, month, date, hrs, min)`
        or `GregorianCalendar(year + 1900, month, date, hrs, min)`.
        """
        ...


    def __init__(self, year: int, month: int, date: int, hrs: int, min: int, sec: int):
        """
        Allocates a `Date` object and initializes it so that
        it represents the instant at the start of the second specified
        by the `year`, `month`, `date`,
        `hrs`, `min`, and `sec` arguments,
        in the local time zone.

        Arguments
        - year: the year minus 1900.
        - month: the month between 0-11.
        - date: the day of the month between 1-31.
        - hrs: the hours between 0-23.
        - min: the minutes between 0-59.
        - sec: the seconds between 0-59.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.set(year + 1900, month, date, hrs, min, sec)`
        or `GregorianCalendar(year + 1900, month, date, hrs, min, sec)`.
        """
        ...


    def __init__(self, s: str):
        """
        Allocates a `Date` object and initializes it so that
        it represents the date and time indicated by the string
        `s`, which is interpreted as if by the
        Date.parse method.

        Arguments
        - s: a string representation of the date.

        See
        - java.util.Date.parse(java.lang.String)

        Deprecated
        - As of JDK version 1.1,
        replaced by `DateFormat.parse(String s)`.
        """
        ...


    def clone(self) -> "Object":
        """
        Return a copy of this object.
        """
        ...


    @staticmethod
    def UTC(year: int, month: int, date: int, hrs: int, min: int, sec: int) -> int:
        """
        Determines the date and time based on the arguments. The
        arguments are interpreted as a year, month, day of the month,
        hour of the day, minute within the hour, and second within the
        minute, exactly as for the `Date` constructor with six
        arguments, except that the arguments are interpreted relative
        to UTC rather than to the local time zone. The time indicated is
        returned represented as the distance, measured in milliseconds,
        of that time from the epoch (00:00:00 GMT on January 1, 1970).

        Arguments
        - year: the year minus 1900.
        - month: the month between 0-11.
        - date: the day of the month between 1-31.
        - hrs: the hours between 0-23.
        - min: the minutes between 0-59.
        - sec: the seconds between 0-59.

        Returns
        - the number of milliseconds since January 1, 1970, 00:00:00 GMT for
                 the date and time specified by the arguments.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.set(year + 1900, month, date, hrs, min, sec)`
        or `GregorianCalendar(year + 1900, month, date, hrs, min, sec)`, using a UTC
        `TimeZone`, followed by `Calendar.getTime().getTime()`.
        """
        ...


    @staticmethod
    def parse(s: str) -> int:
        """
        Attempts to interpret the string `s` as a representation
        of a date and time. If the attempt is successful, the time
        indicated is returned represented as the distance, measured in
        milliseconds, of that time from the epoch (00:00:00 GMT on
        January 1, 1970). If the attempt fails, an
        `IllegalArgumentException` is thrown.
        
        It accepts many syntaxes; in particular, it recognizes the IETF
        standard date syntax: "Sat, 12 Aug 1995 13:30:00 GMT". It also
        understands the continental U.S. time-zone abbreviations, but for
        general use, a time-zone offset should be used: "Sat, 12 Aug 1995
        13:30:00 GMT+0430" (4 hours, 30 minutes west of the Greenwich
        meridian). If no time zone is specified, the local time zone is
        assumed. GMT and UTC are considered equivalent.
        
        The string `s` is processed from left to right, looking for
        data of interest. Any material in `s` that is within the
        ASCII parenthesis characters `(` and `)` is ignored.
        Parentheses may be nested. Otherwise, the only characters permitted
        within `s` are these ASCII characters:
        <blockquote>```
        abcdefghijklmnopqrstuvwxyz
        ABCDEFGHIJKLMNOPQRSTUVWXYZ
        0123456789,+-:/```</blockquote>
        and whitespace characters.
        A consecutive sequence of decimal digits is treated as a decimal
        number:
        - If a number is preceded by `+` or `-` and a year
            has already been recognized, then the number is a time-zone
            offset. If the number is less than 24, it is an offset measured
            in hours. Otherwise, it is regarded as an offset in minutes,
            expressed in 24-hour time format without punctuation. A
            preceding `-` means a westward offset. Time zone offsets
            are always relative to UTC (Greenwich). Thus, for example,
            `-5` occurring in the string would mean "five hours west
            of Greenwich" and `+0430` would mean "four hours and
            thirty minutes east of Greenwich." It is permitted for the
            string to specify `GMT`, `UT`, or `UTC`
            redundantly-for example, `GMT-5` or `utc+0430`.
        - The number is regarded as a year number if one of the
            following conditions is True:
        
            - The number is equal to or greater than 70 and followed by a
                space, comma, slash, or end of string
            - The number is less than 70, and both a month and a day of
                the month have already been recognized
        
            If the recognized year number is less than 100, it is
            interpreted as an abbreviated year relative to a century of
            which dates are within 80 years before and 19 years after
            the time when the Date class is initialized.
            After adjusting the year number, 1900 is subtracted from
            it. For example, if the current year is 1999 then years in
            the range 19 to 99 are assumed to mean 1919 to 1999, while
            years from 0 to 18 are assumed to mean 2000 to 2018.  Note
            that this is slightly different from the interpretation of
            years less than 100 that is used in java.text.SimpleDateFormat.
        - If the number is followed by a colon, it is regarded as an hour,
            unless an hour has already been recognized, in which case it is
            regarded as a minute.
        - If the number is followed by a slash, it is regarded as a month
            (it is decreased by 1 to produce a number in the range `0`
            to `11`), unless a month has already been recognized, in
            which case it is regarded as a day of the month.
        - If the number is followed by whitespace, a comma, a hyphen, or
            end of string, then if an hour has been recognized but not a
            minute, it is regarded as a minute; otherwise, if a minute has
            been recognized but not a second, it is regarded as a second;
            otherwise, it is regarded as a day of the month. 
        A consecutive sequence of letters is regarded as a word and treated
        as follows:
        - A word that matches `AM`, ignoring case, is ignored (but
            the parse fails if an hour has not been recognized or is less
            than `1` or greater than `12`).
        - A word that matches `PM`, ignoring case, adds `12`
            to the hour (but the parse fails if an hour has not been
            recognized or is less than `1` or greater than `12`).
        - Any word that matches any prefix of `SUNDAY, MONDAY, TUESDAY,
            WEDNESDAY, THURSDAY, FRIDAY`, or `SATURDAY`, ignoring
            case, is ignored. For example, `sat, Friday, TUE`, and
            `Thurs` are ignored.
        - Otherwise, any word that matches any prefix of `JANUARY,
            FEBRUARY, MARCH, APRIL, MAY, JUNE, JULY, AUGUST, SEPTEMBER,
            OCTOBER, NOVEMBER`, or `DECEMBER`, ignoring case, and
            considering them in the order given here, is recognized as
            specifying a month and is converted to a number (`0` to
            `11`). For example, `aug, Sept, april`, and
            `NOV` are recognized as months. So is `Ma`, which
            is recognized as `MARCH`, not `MAY`.
        - Any word that matches `GMT, UT`, or `UTC`, ignoring
            case, is treated as referring to UTC.
        - Any word that matches `EST, CST, MST`, or `PST`,
            ignoring case, is recognized as referring to the time zone in
            North America that is five, six, seven, or eight hours west of
            Greenwich, respectively. Any word that matches `EDT, CDT,
            MDT`, or `PDT`, ignoring case, is recognized as
            referring to the same time zone, respectively, during daylight
            saving time.
        Once the entire string s has been scanned, it is converted to a time
        result in one of two ways. If a time zone or time-zone offset has been
        recognized, then the year, month, day of month, hour, minute, and
        second are interpreted in UTC and then the time-zone offset is
        applied. Otherwise, the year, month, day of month, hour, minute, and
        second are interpreted in the local time zone.

        Arguments
        - s: a string to be parsed as a date.

        Returns
        - the number of milliseconds since January 1, 1970, 00:00:00 GMT
                 represented by the string argument.

        See
        - java.text.DateFormat

        Deprecated
        - As of JDK version 1.1,
        replaced by `DateFormat.parse(String s)`.
        """
        ...


    def getYear(self) -> int:
        """
        Returns a value that is the result of subtracting 1900 from the
        year that contains or begins with the instant in time represented
        by this `Date` object, as interpreted in the local
        time zone.

        Returns
        - the year represented by this date, minus 1900.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.get(Calendar.YEAR) - 1900`.
        """
        ...


    def setYear(self, year: int) -> None:
        """
        Sets the year of this `Date` object to be the specified
        value plus 1900. This `Date` object is modified so
        that it represents a point in time within the specified year,
        with the month, date, hour, minute, and second the same as
        before, as interpreted in the local time zone. (Of course, if
        the date was February 29, for example, and the year is set to a
        non-leap year, then the new date will be treated as if it were
        on March 1.)

        Arguments
        - year: the year value.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.set(Calendar.YEAR, year + 1900)`.
        """
        ...


    def getMonth(self) -> int:
        """
        Returns a number representing the month that contains or begins
        with the instant in time represented by this `Date` object.
        The value returned is between `0` and `11`,
        with the value `0` representing January.

        Returns
        - the month represented by this date.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.get(Calendar.MONTH)`.
        """
        ...


    def setMonth(self, month: int) -> None:
        """
        Sets the month of this date to the specified value. This
        `Date` object is modified so that it represents a point
        in time within the specified month, with the year, date, hour,
        minute, and second the same as before, as interpreted in the
        local time zone. If the date was October 31, for example, and
        the month is set to June, then the new date will be treated as
        if it were on July 1, because June has only 30 days.

        Arguments
        - month: the month value between 0-11.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.set(Calendar.MONTH, int month)`.
        """
        ...


    def getDate(self) -> int:
        """
        Returns the day of the month represented by this `Date` object.
        The value returned is between `1` and `31`
        representing the day of the month that contains or begins with the
        instant in time represented by this `Date` object, as
        interpreted in the local time zone.

        Returns
        - the day of the month represented by this date.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.get(Calendar.DAY_OF_MONTH)`.
        """
        ...


    def setDate(self, date: int) -> None:
        """
        Sets the day of the month of this `Date` object to the
        specified value. This `Date` object is modified so that
        it represents a point in time within the specified day of the
        month, with the year, month, hour, minute, and second the same
        as before, as interpreted in the local time zone. If the date
        was April 30, for example, and the date is set to 31, then it
        will be treated as if it were on May 1, because April has only
        30 days.

        Arguments
        - date: the day of the month value between 1-31.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.set(Calendar.DAY_OF_MONTH, int date)`.
        """
        ...


    def getDay(self) -> int:
        """
        Returns the day of the week represented by this date. The
        returned value (`0` = Sunday, `1` = Monday,
        `2` = Tuesday, `3` = Wednesday, `4` =
        Thursday, `5` = Friday, `6` = Saturday)
        represents the day of the week that contains or begins with
        the instant in time represented by this `Date` object,
        as interpreted in the local time zone.

        Returns
        - the day of the week represented by this date.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.get(Calendar.DAY_OF_WEEK)`.
        """
        ...


    def getHours(self) -> int:
        """
        Returns the hour represented by this `Date` object. The
        returned value is a number (`0` through `23`)
        representing the hour within the day that contains or begins
        with the instant in time represented by this `Date`
        object, as interpreted in the local time zone.

        Returns
        - the hour represented by this date.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.get(Calendar.HOUR_OF_DAY)`.
        """
        ...


    def setHours(self, hours: int) -> None:
        """
        Sets the hour of this `Date` object to the specified value.
        This `Date` object is modified so that it represents a point
        in time within the specified hour of the day, with the year, month,
        date, minute, and second the same as before, as interpreted in the
        local time zone.

        Arguments
        - hours: the hour value.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.set(Calendar.HOUR_OF_DAY, int hours)`.
        """
        ...


    def getMinutes(self) -> int:
        """
        Returns the number of minutes past the hour represented by this date,
        as interpreted in the local time zone.
        The value returned is between `0` and `59`.

        Returns
        - the number of minutes past the hour represented by this date.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.get(Calendar.MINUTE)`.
        """
        ...


    def setMinutes(self, minutes: int) -> None:
        """
        Sets the minutes of this `Date` object to the specified value.
        This `Date` object is modified so that it represents a point
        in time within the specified minute of the hour, with the year, month,
        date, hour, and second the same as before, as interpreted in the
        local time zone.

        Arguments
        - minutes: the value of the minutes.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.set(Calendar.MINUTE, int minutes)`.
        """
        ...


    def getSeconds(self) -> int:
        """
        Returns the number of seconds past the minute represented by this date.
        The value returned is between `0` and `61`. The
        values `60` and `61` can only occur on those
        Java Virtual Machines that take leap seconds into account.

        Returns
        - the number of seconds past the minute represented by this date.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.get(Calendar.SECOND)`.
        """
        ...


    def setSeconds(self, seconds: int) -> None:
        """
        Sets the seconds of this `Date` to the specified value.
        This `Date` object is modified so that it represents a
        point in time within the specified second of the minute, with
        the year, month, date, hour, and minute the same as before, as
        interpreted in the local time zone.

        Arguments
        - seconds: the seconds value.

        See
        - java.util.Calendar

        Deprecated
        - As of JDK version 1.1,
        replaced by `Calendar.set(Calendar.SECOND, int seconds)`.
        """
        ...


    def getTime(self) -> int:
        """
        Returns the number of milliseconds since January 1, 1970, 00:00:00 GMT
        represented by this `Date` object.

        Returns
        - the number of milliseconds since January 1, 1970, 00:00:00 GMT
                 represented by this date.
        """
        ...


    def setTime(self, time: int) -> None:
        """
        Sets this `Date` object to represent a point in time that is
        `time` milliseconds after January 1, 1970 00:00:00 GMT.

        Arguments
        - time: the number of milliseconds.
        """
        ...


    def before(self, when: "Date") -> bool:
        """
        Tests if this date is before the specified date.

        Arguments
        - when: a date.

        Returns
        - `True` if and only if the instant of time
                   represented by this `Date` object is strictly
                   earlier than the instant represented by `when`;
                 `False` otherwise.

        Raises
        - NullPointerException: if `when` is null.
        """
        ...


    def after(self, when: "Date") -> bool:
        """
        Tests if this date is after the specified date.

        Arguments
        - when: a date.

        Returns
        - `True` if and only if the instant represented
                 by this `Date` object is strictly later than the
                 instant represented by `when`;
                 `False` otherwise.

        Raises
        - NullPointerException: if `when` is null.
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Compares two dates for equality.
        The result is `True` if and only if the argument is
        not `null` and is a `Date` object that
        represents the same point in time, to the millisecond, as this object.
        
        Thus, two `Date` objects are equal if and only if the
        `getTime` method returns the same `long`
        value for both.

        Arguments
        - obj: the object to compare with.

        Returns
        - `True` if the objects are the same;
                 `False` otherwise.

        See
        - java.util.Date.getTime()
        """
        ...


    def compareTo(self, anotherDate: "Date") -> int:
        """
        Compares two Dates for ordering.

        Arguments
        - anotherDate: the `Date` to be compared.

        Returns
        - the value `0` if the argument Date is equal to
                 this Date; a value less than `0` if this Date
                 is before the Date argument; and a value greater than
             `0` if this Date is after the Date argument.

        Raises
        - NullPointerException: if `anotherDate` is null.

        Since
        - 1.2
        """
        ...


    def hashCode(self) -> int:
        """
        Returns a hash code value for this object. The result is the
        exclusive OR of the two halves of the primitive `long`
        value returned by the Date.getTime
        method. That is, the hash code is the value of the expression:
        <blockquote>````(int)(this.getTime()^(this.getTime() >>> 32))````</blockquote>

        Returns
        - a hash code value for this object.
        """
        ...


    def toString(self) -> str:
        """
        Converts this `Date` object to a `String`
        of the form:
        <blockquote>```
        dow mon dd hh:mm:ss zzz yyyy```</blockquote>
        where:
        - `dow` is the day of the week (`Sun, Mon, Tue, Wed,
            Thu, Fri, Sat`).
        - `mon` is the month (`Jan, Feb, Mar, Apr, May, Jun,
            Jul, Aug, Sep, Oct, Nov, Dec`).
        - `dd` is the day of the month (`01` through
            `31`), as two decimal digits.
        - `hh` is the hour of the day (`00` through
            `23`), as two decimal digits.
        - `mm` is the minute within the hour (`00` through
            `59`), as two decimal digits.
        - `ss` is the second within the minute (`00` through
            `61`, as two decimal digits.
        - `zzz` is the time zone (and may reflect daylight saving
            time). Standard time zone abbreviations include those
            recognized by the method `parse`. If time zone
            information is not available, then `zzz` is empty -
            that is, it consists of no characters at all.
        - `yyyy` is the year, as four decimal digits.

        Returns
        - a string representation of this date.

        See
        - java.util.Date.toGMTString()
        """
        ...


    def toLocaleString(self) -> str:
        """
        Creates a string representation of this `Date` object in an
        implementation-dependent form. The intent is that the form should
        be familiar to the user of the Java application, wherever it may
        happen to be running. The intent is comparable to that of the
        "`%c`" format supported by the `strftime()`
        function of ISO&nbsp;C.

        Returns
        - a string representation of this date, using the locale
                 conventions.

        See
        - java.util.Date.toGMTString()

        Deprecated
        - As of JDK version 1.1,
        replaced by `DateFormat.format(Date date)`.
        """
        ...


    def toGMTString(self) -> str:
        """
        Creates a string representation of this `Date` object of
        the form:
        <blockquote>```
        d mon yyyy hh:mm:ss GMT```</blockquote>
        where:
        - *d* is the day of the month (`1` through `31`),
            as one or two decimal digits.
        - *mon* is the month (`Jan, Feb, Mar, Apr, May, Jun, Jul,
            Aug, Sep, Oct, Nov, Dec`).
        - *yyyy* is the year, as four decimal digits.
        - *hh* is the hour of the day (`00` through `23`),
            as two decimal digits.
        - *mm* is the minute within the hour (`00` through
            `59`), as two decimal digits.
        - *ss* is the second within the minute (`00` through
            `61`), as two decimal digits.
        - *GMT* is exactly the ASCII letters "`GMT`" to indicate
            Greenwich Mean Time.
        
        The result does not depend on the local time zone.

        Returns
        - a string representation of this date, using the Internet GMT
                 conventions.

        See
        - java.util.Date.toLocaleString()

        Deprecated
        - As of JDK version 1.1,
        replaced by `DateFormat.format(Date date)`, using a
        GMT `TimeZone`.
        """
        ...


    def getTimezoneOffset(self) -> int:
        """
        Returns the offset, measured in minutes, for the local time zone
        relative to UTC that is appropriate for the time represented by
        this `Date` object.
        
        For example, in Massachusetts, five time zones west of Greenwich:
        <blockquote>```
        new Date(96, 1, 14).getTimezoneOffset() returns 300```</blockquote>
        because on February 14, 1996, standard time (Eastern Standard Time)
        is in use, which is offset five hours from UTC; but:
        <blockquote>```
        new Date(96, 5, 1).getTimezoneOffset() returns 240```</blockquote>
        because on June 1, 1996, daylight saving time (Eastern Daylight Time)
        is in use, which is offset only four hours from UTC.
        This method produces the same result as if it computed:
        <blockquote>```
        (this.getTime() - UTC(this.getYear(),
                              this.getMonth(),
                              this.getDate(),
                              this.getHours(),
                              this.getMinutes(),
                              this.getSeconds())) / (60 * 1000)
        ```</blockquote>

        Returns
        - the time-zone offset, in minutes, for the current time zone.

        See
        - java.util.TimeZone.getDefault

        Deprecated
        - As of JDK version 1.1,
        replaced by `-(Calendar.get(Calendar.ZONE_OFFSET) +
        Calendar.get(Calendar.DST_OFFSET)) / (60 * 1000)`.
        """
        ...


    @staticmethod
    def from(instant: "Instant") -> "Date":
        """
        Obtains an instance of `Date` from an `Instant` object.
        
        `Instant` uses a precision of nanoseconds, whereas `Date`
        uses a precision of milliseconds.  The conversion will truncate any
        excess precision information as though the amount in nanoseconds was
        subject to integer division by one million.
        
        `Instant` can store points on the time-line further in the future
        and further in the past than `Date`. In this scenario, this method
        will throw an exception.

        Arguments
        - instant: the instant to convert

        Returns
        - a `Date` representing the same point on the time-line as
         the provided instant

        Raises
        - NullPointerException: if `instant` is null.
        - IllegalArgumentException: if the instant is too large to
         represent as a `Date`

        Since
        - 1.8
        """
        ...


    def toInstant(self) -> "Instant":
        """
        Converts this `Date` object to an `Instant`.
        
        The conversion creates an `Instant` that represents the same
        point on the time-line as this `Date`.

        Returns
        - an instant representing the same point on the time-line as
         this `Date` object

        Since
        - 1.8
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.Maps

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Converter
from com.google.common.base import Equivalence
from com.google.common.base import Function
from com.google.common.base import Objects
from com.google.common.base import Preconditions
from com.google.common.base import Predicate
from com.google.common.base import Predicates
from com.google.common.collect import *
from com.google.common.collect.MapDifference import ValueDifference
from com.google.common.primitives import Ints
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations.concurrent import LazyInit
from com.google.j2objc.annotations import RetainedWith
from com.google.j2objc.annotations import Weak
from com.google.j2objc.annotations import WeakOuter
from java.io import Serializable
from java.util import AbstractCollection
from java.util import Collections
from java.util import Comparator
from java.util import EnumMap
from java.util import Enumeration
from java.util import IdentityHashMap
from java.util import Iterator
from java.util import NavigableMap
from java.util import NavigableSet
from java.util import Properties
from java.util import SortedMap
from java.util import SortedSet
from java.util import Spliterator
from java.util.concurrent import ConcurrentHashMap
from java.util.concurrent import ConcurrentMap
from java.util.function import BiConsumer
from java.util.function import BiFunction
from java.util.function import BinaryOperator
from java.util.function import Consumer
from java.util.stream import Collector
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import NonNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Maps:
    """
    Static utility methods pertaining to Map instances (including instances of SortedMap, BiMap, etc.). Also see this class's counterparts Lists, Sets
    and Queues.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/CollectionUtilitiesExplained#maps">`Maps`</a>.

    Author(s)
    - Louis Wasserman

    Since
    - 2.0
    """

    @staticmethod
    def immutableEnumMap(map: dict["K", "V"]) -> "ImmutableMap"["K", "V"]:
        """
        Returns an immutable map instance containing the given entries. Internally, the returned map
        will be backed by an EnumMap.
        
        The iteration order of the returned map follows the enum's iteration order, not the order in
        which the elements appear in the given map.

        Arguments
        - map: the map to make an immutable copy of

        Returns
        - an immutable map containing those entries

        Since
        - 14.0
        """
        ...


    @staticmethod
    def toImmutableEnumMap(keyFunction: "java.util.function.Function"["T", "K"], valueFunction: "java.util.function.Function"["T", "V"]) -> "Collector"["T", Any, "ImmutableMap"["K", "V"]]:
        """
        Returns a Collector that accumulates elements into an `ImmutableMap` whose keys
        and values are the result of applying the provided mapping functions to the input elements. The
        resulting implementation is specialized for enum key types. The returned map and its views will
        iterate over keys in their enum definition order, not encounter order.
        
        If the mapped keys contain duplicates, an `IllegalArgumentException` is thrown when
        the collection operation is performed. (This differs from the `Collector` returned by
        java.util.stream.Collectors.toMap(java.util.function.Function,
        java.util.function.Function) Collectors.toMap(Function, Function), which throws an `IllegalStateException`.)

        Since
        - 21.0
        """
        ...


    @staticmethod
    def toImmutableEnumMap(keyFunction: "java.util.function.Function"["T", "K"], valueFunction: "java.util.function.Function"["T", "V"], mergeFunction: "BinaryOperator"["V"]) -> "Collector"["T", Any, "ImmutableMap"["K", "V"]]:
        """
        Returns a Collector that accumulates elements into an `ImmutableMap` whose keys
        and values are the result of applying the provided mapping functions to the input elements. The
        resulting implementation is specialized for enum key types. The returned map and its views will
        iterate over keys in their enum definition order, not encounter order.
        
        If the mapped keys contain duplicates, the values are merged using the specified merging
        function.

        Since
        - 21.0
        """
        ...


    @staticmethod
    def newHashMap() -> dict["K", "V"]:
        """
        Creates a *mutable*, empty `HashMap` instance.
        
        **Note:** if mutability is not required, use ImmutableMap.of() instead.
        
        **Note:** if `K` is an `enum` type, use .newEnumMap instead.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `HashMap` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.

        Returns
        - a new, empty `HashMap`
        """
        ...


    @staticmethod
    def newHashMap(map: dict["K", "V"]) -> dict["K", "V"]:
        """
        Creates a *mutable* `HashMap` instance with the same mappings as the specified map.
        
        **Note:** if mutability is not required, use ImmutableMap.copyOf(Map) instead.
        
        **Note:** if `K` is an Enum type, use .newEnumMap instead.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `HashMap` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.

        Arguments
        - map: the mappings to be placed in the new map

        Returns
        - a new `HashMap` initialized with the mappings from `map`
        """
        ...


    @staticmethod
    def newHashMapWithExpectedSize(expectedSize: int) -> dict["K", "V"]:
        """
        Creates a `HashMap` instance, with a high enough "initial capacity" that it *should*
        hold `expectedSize` elements without growth. This behavior cannot be broadly guaranteed,
        but it is observed to be True for OpenJDK 1.7. It also can't be guaranteed that the method
        isn't inadvertently *oversizing* the returned map.

        Arguments
        - expectedSize: the number of entries you expect to add to the returned map

        Returns
        - a new, empty `HashMap` with enough capacity to hold `expectedSize` entries
            without resizing

        Raises
        - IllegalArgumentException: if `expectedSize` is negative
        """
        ...


    @staticmethod
    def newLinkedHashMap() -> dict["K", "V"]:
        """
        Creates a *mutable*, empty, insertion-ordered `LinkedHashMap` instance.
        
        **Note:** if mutability is not required, use ImmutableMap.of() instead.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `LinkedHashMap` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.

        Returns
        - a new, empty `LinkedHashMap`
        """
        ...


    @staticmethod
    def newLinkedHashMap(map: dict["K", "V"]) -> dict["K", "V"]:
        """
        Creates a *mutable*, insertion-ordered `LinkedHashMap` instance with the same
        mappings as the specified map.
        
        **Note:** if mutability is not required, use ImmutableMap.copyOf(Map) instead.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `LinkedHashMap` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.

        Arguments
        - map: the mappings to be placed in the new map

        Returns
        - a new, `LinkedHashMap` initialized with the mappings from `map`
        """
        ...


    @staticmethod
    def newLinkedHashMapWithExpectedSize(expectedSize: int) -> dict["K", "V"]:
        """
        Creates a `LinkedHashMap` instance, with a high enough "initial capacity" that it
        *should* hold `expectedSize` elements without growth. This behavior cannot be
        broadly guaranteed, but it is observed to be True for OpenJDK 1.7. It also can't be guaranteed
        that the method isn't inadvertently *oversizing* the returned map.

        Arguments
        - expectedSize: the number of entries you expect to add to the returned map

        Returns
        - a new, empty `LinkedHashMap` with enough capacity to hold `expectedSize`
            entries without resizing

        Raises
        - IllegalArgumentException: if `expectedSize` is negative

        Since
        - 19.0
        """
        ...


    @staticmethod
    def newConcurrentMap() -> "ConcurrentMap"["K", "V"]:
        """
        Creates a new empty ConcurrentHashMap instance.

        Since
        - 3.0
        """
        ...


    @staticmethod
    def newTreeMap() -> dict["K", "V"]:
        """
        Creates a *mutable*, empty `TreeMap` instance using the natural ordering of its
        elements.
        
        **Note:** if mutability is not required, use ImmutableSortedMap.of() instead.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `TreeMap` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.

        Returns
        - a new, empty `TreeMap`
        """
        ...


    @staticmethod
    def newTreeMap(map: "SortedMap"["K", "V"]) -> dict["K", "V"]:
        """
        Creates a *mutable* `TreeMap` instance with the same mappings as the specified map
        and using the same ordering as the specified map.
        
        **Note:** if mutability is not required, use ImmutableSortedMap.copyOfSorted(SortedMap) instead.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `TreeMap` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.

        Arguments
        - map: the sorted map whose mappings are to be placed in the new map and whose comparator
            is to be used to sort the new map

        Returns
        - a new `TreeMap` initialized with the mappings from `map` and using the
            comparator of `map`
        """
        ...


    @staticmethod
    def newTreeMap(comparator: "Comparator"["C"]) -> dict["K", "V"]:
        """
        Creates a *mutable*, empty `TreeMap` instance using the given comparator.
        
        **Note:** if mutability is not required, use `ImmutableSortedMap.orderedBy(comparator).build()` instead.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `TreeMap` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.

        Arguments
        - comparator: the comparator to sort the keys with

        Returns
        - a new, empty `TreeMap`
        """
        ...


    @staticmethod
    def newEnumMap(type: type["K"]) -> "EnumMap"["K", "V"]:
        """
        Creates an `EnumMap` instance.

        Arguments
        - type: the key type for this map

        Returns
        - a new, empty `EnumMap`
        """
        ...


    @staticmethod
    def newEnumMap(map: dict["K", "V"]) -> "EnumMap"["K", "V"]:
        """
        Creates an `EnumMap` with the same mappings as the specified map.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `EnumMap` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.

        Arguments
        - map: the map from which to initialize this `EnumMap`

        Returns
        - a new `EnumMap` initialized with the mappings from `map`

        Raises
        - IllegalArgumentException: if `m` is not an `EnumMap` instance and contains
            no mappings
        """
        ...


    @staticmethod
    def newIdentityHashMap() -> "IdentityHashMap"["K", "V"]:
        """
        Creates an `IdentityHashMap` instance.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `IdentityHashMap` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.

        Returns
        - a new, empty `IdentityHashMap`
        """
        ...


    @staticmethod
    def difference(left: dict["K", "V"], right: dict["K", "V"]) -> "MapDifference"["K", "V"]:
        """
        Computes the difference between two maps. This difference is an immutable snapshot of the state
        of the maps at the time this method is called. It will never change, even if the maps change at
        a later time.
        
        Since this method uses `HashMap` instances internally, the keys of the supplied maps
        must be well-behaved with respect to Object.equals and Object.hashCode.
        
        **Note:**If you only need to know whether two maps have the same mappings, call `left.equals(right)` instead of this method.

        Arguments
        - left: the map to treat as the "left" map for purposes of comparison
        - right: the map to treat as the "right" map for purposes of comparison

        Returns
        - the difference between the two maps
        """
        ...


    @staticmethod
    def difference(left: dict["K", "V"], right: dict["K", "V"], valueEquivalence: "Equivalence"["V"]) -> "MapDifference"["K", "V"]:
        """
        Computes the difference between two maps. This difference is an immutable snapshot of the state
        of the maps at the time this method is called. It will never change, even if the maps change at
        a later time.
        
        Since this method uses `HashMap` instances internally, the keys of the supplied maps
        must be well-behaved with respect to Object.equals and Object.hashCode.

        Arguments
        - left: the map to treat as the "left" map for purposes of comparison
        - right: the map to treat as the "right" map for purposes of comparison
        - valueEquivalence: the equivalence relationship to use to compare values

        Returns
        - the difference between the two maps

        Since
        - 10.0
        """
        ...


    @staticmethod
    def difference(left: "SortedMap"["K", "V"], right: dict["K", "V"]) -> "SortedMapDifference"["K", "V"]:
        """
        Computes the difference between two sorted maps, using the comparator of the left map, or
        `Ordering.natural()` if the left map uses the natural ordering of its elements. This
        difference is an immutable snapshot of the state of the maps at the time this method is called.
        It will never change, even if the maps change at a later time.
        
        Since this method uses `TreeMap` instances internally, the keys of the right map must
        all compare as distinct according to the comparator of the left map.
        
        **Note:**If you only need to know whether two sorted maps have the same mappings, call
        `left.equals(right)` instead of this method.

        Arguments
        - left: the map to treat as the "left" map for purposes of comparison
        - right: the map to treat as the "right" map for purposes of comparison

        Returns
        - the difference between the two maps

        Since
        - 11.0
        """
        ...


    @staticmethod
    def asMap(set: set["K"], function: "Function"["K", "V"]) -> dict["K", "V"]:
        """
        Returns a live Map view whose keys are the contents of `set` and whose values are
        computed on demand using `function`. To get an immutable *copy* instead, use .toMap(Iterable, Function).
        
        Specifically, for each `k` in the backing set, the returned map has an entry mapping
        `k` to `function.apply(k)`. The `keySet`, `values`, and `entrySet` views of the returned map iterate in the same order as the backing set.
        
        Modifications to the backing set are read through to the returned map. The returned map
        supports removal operations if the backing set does. Removal operations write through to the
        backing set. The returned map does not support put operations.
        
        **Warning:** If the function rejects `null`, caution is required to make sure the
        set does not contain `null`, because the view cannot stop `null` from being added
        to the set.
        
        **Warning:** This method assumes that for any instance `k` of key type `K`,
        `k.equals(k2)` implies that `k2` is also of type `K`. Using a key type for
        which this may not hold, such as `ArrayList`, may risk a `ClassCastException` when
        calling methods on the resulting map view.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def asMap(set: "SortedSet"["K"], function: "Function"["K", "V"]) -> "SortedMap"["K", "V"]:
        """
        Returns a view of the sorted set as a map, mapping keys from the set according to the specified
        function.
        
        Specifically, for each `k` in the backing set, the returned map has an entry mapping
        `k` to `function.apply(k)`. The `keySet`, `values`, and `entrySet` views of the returned map iterate in the same order as the backing set.
        
        Modifications to the backing set are read through to the returned map. The returned map
        supports removal operations if the backing set does. Removal operations write through to the
        backing set. The returned map does not support put operations.
        
        **Warning:** If the function rejects `null`, caution is required to make sure the
        set does not contain `null`, because the view cannot stop `null` from being added
        to the set.
        
        **Warning:** This method assumes that for any instance `k` of key type `K`,
        `k.equals(k2)` implies that `k2` is also of type `K`. Using a key type for
        which this may not hold, such as `ArrayList`, may risk a `ClassCastException` when
        calling methods on the resulting map view.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def asMap(set: "NavigableSet"["K"], function: "Function"["K", "V"]) -> "NavigableMap"["K", "V"]:
        """
        Returns a view of the navigable set as a map, mapping keys from the set according to the
        specified function.
        
        Specifically, for each `k` in the backing set, the returned map has an entry mapping
        `k` to `function.apply(k)`. The `keySet`, `values`, and `entrySet` views of the returned map iterate in the same order as the backing set.
        
        Modifications to the backing set are read through to the returned map. The returned map
        supports removal operations if the backing set does. Removal operations write through to the
        backing set. The returned map does not support put operations.
        
        **Warning:** If the function rejects `null`, caution is required to make sure the
        set does not contain `null`, because the view cannot stop `null` from being added
        to the set.
        
        **Warning:** This method assumes that for any instance `k` of key type `K`,
        `k.equals(k2)` implies that `k2` is also of type `K`. Using a key type for
        which this may not hold, such as `ArrayList`, may risk a `ClassCastException` when
        calling methods on the resulting map view.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def toMap(keys: Iterable["K"], valueFunction: "Function"["K", "V"]) -> "ImmutableMap"["K", "V"]:
        """
        Returns an immutable map whose keys are the distinct elements of `keys` and whose value
        for each key was computed by `valueFunction`. The map's iteration order is the order of
        the first appearance of each key in `keys`.
        
        When there are multiple instances of a key in `keys`, it is unspecified whether `valueFunction` will be applied to more than one instance of that key and, if it is, which
        result will be mapped to that key in the returned map.
        
        If `keys` is a Set, a live view can be obtained instead of a copy using Maps.asMap(Set, Function).
        
        **Note:** on Java 8+, it is usually better to use streams. For example:
        
        ````import static com.google.common.collect.ImmutableMap.toImmutableMap;
        ...
        ImmutableMap<Color, String> colorNames =
            allColors.stream().collect(toImmutableMap(c -> c, c -> c.toString()));````
        
        Streams provide a more standard and flexible API and the lambdas make it clear what the keys
        and values in the map are.

        Raises
        - NullPointerException: if any element of `keys` is `null`, or if `valueFunction` produces `null` for any key

        Since
        - 14.0
        """
        ...


    @staticmethod
    def toMap(keys: Iterator["K"], valueFunction: "Function"["K", "V"]) -> "ImmutableMap"["K", "V"]:
        """
        Returns an immutable map whose keys are the distinct elements of `keys` and whose value
        for each key was computed by `valueFunction`. The map's iteration order is the order of
        the first appearance of each key in `keys`.
        
        When there are multiple instances of a key in `keys`, it is unspecified whether `valueFunction` will be applied to more than one instance of that key and, if it is, which
        result will be mapped to that key in the returned map.

        Raises
        - NullPointerException: if any element of `keys` is `null`, or if `valueFunction` produces `null` for any key

        Since
        - 14.0
        """
        ...


    @staticmethod
    def uniqueIndex(values: Iterable["V"], keyFunction: "Function"["V", "K"]) -> "ImmutableMap"["K", "V"]:
        """
        Returns a map with the given `values`, indexed by keys derived from those values. In
        other words, each input value produces an entry in the map whose key is the result of applying
        `keyFunction` to that value. These entries appear in the same order as the input values.
        Example usage:
        
        ````Color red = new Color("red", 255, 0, 0);
        ...
        ImmutableSet<Color> allColors = ImmutableSet.of(red, green, blue);
        
        ImmutableMap<String, Color> colorForName =
            uniqueIndex(allColors, c -> c.toString());
        assertThat(colorForName).containsEntry("red", red);````
        
        If your index may associate multiple values with each key, use Multimaps.index(Iterable, Function) Multimaps.index.
        
        **Note:** on Java 8+, it is usually better to use streams. For example:
        
        ````import static com.google.common.collect.ImmutableMap.toImmutableMap;
        ...
        ImmutableMap<String, Color> colorForName =
            allColors.stream().collect(toImmutableMap(c -> c.toString(), c -> c));````
        
        Streams provide a more standard and flexible API and the lambdas make it clear what the keys
        and values in the map are.

        Arguments
        - values: the values to use when constructing the `Map`
        - keyFunction: the function used to produce the key for each value

        Returns
        - a map mapping the result of evaluating the function `keyFunction` on each value
            in the input collection to that value

        Raises
        - IllegalArgumentException: if `keyFunction` produces the same key for more than one
            value in the input collection
        - NullPointerException: if any element of `values` is `null`, or if `keyFunction` produces `null` for any value
        """
        ...


    @staticmethod
    def uniqueIndex(values: Iterator["V"], keyFunction: "Function"["V", "K"]) -> "ImmutableMap"["K", "V"]:
        """
        Returns a map with the given `values`, indexed by keys derived from those values. In
        other words, each input value produces an entry in the map whose key is the result of applying
        `keyFunction` to that value. These entries appear in the same order as the input values.
        Example usage:
        
        ````Color red = new Color("red", 255, 0, 0);
        ...
        Iterator<Color> allColors = ImmutableSet.of(red, green, blue).iterator();
        
        Map<String, Color> colorForName =
            uniqueIndex(allColors, toStringFunction());
        assertThat(colorForName).containsEntry("red", red);````
        
        If your index may associate multiple values with each key, use Multimaps.index(Iterator, Function) Multimaps.index.

        Arguments
        - values: the values to use when constructing the `Map`
        - keyFunction: the function used to produce the key for each value

        Returns
        - a map mapping the result of evaluating the function `keyFunction` on each value
            in the input collection to that value

        Raises
        - IllegalArgumentException: if `keyFunction` produces the same key for more than one
            value in the input collection
        - NullPointerException: if any element of `values` is `null`, or if `keyFunction` produces `null` for any value

        Since
        - 10.0
        """
        ...


    @staticmethod
    def fromProperties(properties: "Properties") -> "ImmutableMap"[str, str]:
        """
        Creates an `ImmutableMap<String, String>` from a `Properties` instance. Properties
        normally derive from `Map<Object, Object>`, but they typically contain strings, which is
        awkward. This method lets you get a plain-old-`Map` out of a `Properties`.

        Arguments
        - properties: a `Properties` object to be converted

        Returns
        - an immutable map containing all the entries in `properties`

        Raises
        - ClassCastException: if any key in `properties` is not a `String`
        - NullPointerException: if any key or value in `properties` is null
        """
        ...


    @staticmethod
    def immutableEntry(key: "K", value: "V") -> "Entry"["K", "V"]:
        """
        Returns an immutable map entry with the specified key and value. The Entry.setValue
        operation throws an UnsupportedOperationException.
        
        The returned entry is serializable.
        
        **Java 9 users:** consider using `java.util.Map.entry(key, value)` if the key and
        value are non-null and the entry does not need to be serializable.

        Arguments
        - key: the key to be associated with the returned entry
        - value: the value to be associated with the returned entry
        """
        ...


    @staticmethod
    def asConverter(bimap: "BiMap"["A", "B"]) -> "Converter"["A", "B"]:
        """
        Returns a Converter that converts values using BiMap.get bimap.get(), and whose
        inverse view converts values using BiMap.inverse bimap.inverse()`.get()`.
        
        To use a plain Map as a Function, see com.google.common.base.Functions.forMap(Map) or com.google.common.base.Functions.forMap(Map, Object).

        Since
        - 16.0
        """
        ...


    @staticmethod
    def synchronizedBiMap(bimap: "BiMap"["K", "V"]) -> "BiMap"["K", "V"]:
        """
        Returns a synchronized (thread-safe) bimap backed by the specified bimap. In order to guarantee
        serial access, it is critical that **all** access to the backing bimap is accomplished
        through the returned bimap.
        
        It is imperative that the user manually synchronize on the returned map when accessing any
        of its collection views:
        
        ````BiMap<Long, String> map = Maps.synchronizedBiMap(
            HashBiMap.<Long, String>create());
        ...
        Set<Long> set = map.keySet();  // Needn't be in synchronized block
        ...
        synchronized (map) {  // Synchronizing on map, not set!
          Iterator<Long> it = set.iterator(); // Must be in synchronized block
          while (it.hasNext()) {
            foo(it.next());`
        }
        }```
        
        Failure to follow this advice may result in non-deterministic behavior.
        
        The returned bimap will be serializable if the specified bimap is serializable.

        Arguments
        - bimap: the bimap to be wrapped in a synchronized view

        Returns
        - a synchronized view of the specified bimap
        """
        ...


    @staticmethod
    def unmodifiableBiMap(bimap: "BiMap"["K", "V"]) -> "BiMap"["K", "V"]:
        """
        Returns an unmodifiable view of the specified bimap. This method allows modules to provide
        users with "read-only" access to internal bimaps. Query operations on the returned bimap "read
        through" to the specified bimap, and attempts to modify the returned map, whether direct or via
        its collection views, result in an `UnsupportedOperationException`.
        
        The returned bimap will be serializable if the specified bimap is serializable.

        Arguments
        - bimap: the bimap for which an unmodifiable view is to be returned

        Returns
        - an unmodifiable view of the specified bimap
        """
        ...


    @staticmethod
    def transformValues(fromMap: dict["K", "V1"], function: "Function"["V1", "V2"]) -> dict["K", "V2"]:
        """
        Returns a view of a map where each value is transformed by a function. All other properties of
        the map, such as iteration order, are left intact. For example, the code:
        
        ````Map<String, Integer> map = ImmutableMap.of("a", 4, "b", 9);
        Function<Integer, Double> sqrt =
            new Function<Integer, Double>() {
              public Double apply(Integer in) {
                return Math.sqrt((int) in);`
            };
        Map<String, Double> transformed = Maps.transformValues(map, sqrt);
        System.out.println(transformed);
        }```
        
        ... prints `{a=2.0, b=3.0`}.
        
        Changes in the underlying map are reflected in this view. Conversely, this view supports
        removal operations, and these are reflected in the underlying map.
        
        It's acceptable for the underlying map to contain null keys, and even null values provided
        that the function is capable of accepting null input. The transformed map might contain null
        values, if the function sometimes gives a null result.
        
        The returned map is not thread-safe or serializable, even if the underlying map is.
        
        The function is applied lazily, invoked when needed. This is necessary for the returned map
        to be a view, but it means that the function will be applied many times for bulk operations
        like Map.containsValue and `Map.toString()`. For this to perform well, `function` should be fast. To avoid lazy evaluation when the returned map doesn't need to be a
        view, copy the returned map into a new map of your choosing.
        """
        ...


    @staticmethod
    def transformValues(fromMap: "SortedMap"["K", "V1"], function: "Function"["V1", "V2"]) -> "SortedMap"["K", "V2"]:
        """
        Returns a view of a sorted map where each value is transformed by a function. All other
        properties of the map, such as iteration order, are left intact. For example, the code:
        
        ````SortedMap<String, Integer> map = ImmutableSortedMap.of("a", 4, "b", 9);
        Function<Integer, Double> sqrt =
            new Function<Integer, Double>() {
              public Double apply(Integer in) {
                return Math.sqrt((int) in);`
            };
        SortedMap<String, Double> transformed =
             Maps.transformValues(map, sqrt);
        System.out.println(transformed);
        }```
        
        ... prints `{a=2.0, b=3.0`}.
        
        Changes in the underlying map are reflected in this view. Conversely, this view supports
        removal operations, and these are reflected in the underlying map.
        
        It's acceptable for the underlying map to contain null keys, and even null values provided
        that the function is capable of accepting null input. The transformed map might contain null
        values, if the function sometimes gives a null result.
        
        The returned map is not thread-safe or serializable, even if the underlying map is.
        
        The function is applied lazily, invoked when needed. This is necessary for the returned map
        to be a view, but it means that the function will be applied many times for bulk operations
        like Map.containsValue and `Map.toString()`. For this to perform well, `function` should be fast. To avoid lazy evaluation when the returned map doesn't need to be a
        view, copy the returned map into a new map of your choosing.

        Since
        - 11.0
        """
        ...


    @staticmethod
    def transformValues(fromMap: "NavigableMap"["K", "V1"], function: "Function"["V1", "V2"]) -> "NavigableMap"["K", "V2"]:
        """
        Returns a view of a navigable map where each value is transformed by a function. All other
        properties of the map, such as iteration order, are left intact. For example, the code:
        
        ````NavigableMap<String, Integer> map = Maps.newTreeMap();
        map.put("a", 4);
        map.put("b", 9);
        Function<Integer, Double> sqrt =
            new Function<Integer, Double>() {
              public Double apply(Integer in) {
                return Math.sqrt((int) in);`
            };
        NavigableMap<String, Double> transformed =
             Maps.transformNavigableValues(map, sqrt);
        System.out.println(transformed);
        }```
        
        ... prints `{a=2.0, b=3.0`}.
        
        Changes in the underlying map are reflected in this view. Conversely, this view supports
        removal operations, and these are reflected in the underlying map.
        
        It's acceptable for the underlying map to contain null keys, and even null values provided
        that the function is capable of accepting null input. The transformed map might contain null
        values, if the function sometimes gives a null result.
        
        The returned map is not thread-safe or serializable, even if the underlying map is.
        
        The function is applied lazily, invoked when needed. This is necessary for the returned map
        to be a view, but it means that the function will be applied many times for bulk operations
        like Map.containsValue and `Map.toString()`. For this to perform well, `function` should be fast. To avoid lazy evaluation when the returned map doesn't need to be a
        view, copy the returned map into a new map of your choosing.

        Since
        - 13.0
        """
        ...


    @staticmethod
    def transformEntries(fromMap: dict["K", "V1"], transformer: "EntryTransformer"["K", "V1", "V2"]) -> dict["K", "V2"]:
        """
        Returns a view of a map whose values are derived from the original map's entries. In contrast
        to .transformValues, this method's entry-transformation logic may depend on the key as
        well as the value.
        
        All other properties of the transformed map, such as iteration order, are left intact. For
        example, the code:
        
        ````Map<String, Boolean> options =
            ImmutableMap.of("verbose", True, "sort", False);
        EntryTransformer<String, Boolean, String> flagPrefixer =
            new EntryTransformer<String, Boolean, String>() {
              public String transformEntry(String key, Boolean value) {
                return value ? key : "no" + key;`
            };
        Map<String, String> transformed =
            Maps.transformEntries(options, flagPrefixer);
        System.out.println(transformed);
        }```
        
        ... prints `{verbose=verbose, sort=nosort`}.
        
        Changes in the underlying map are reflected in this view. Conversely, this view supports
        removal operations, and these are reflected in the underlying map.
        
        It's acceptable for the underlying map to contain null keys and null values provided that
        the transformer is capable of accepting null inputs. The transformed map might contain null
        values if the transformer sometimes gives a null result.
        
        The returned map is not thread-safe or serializable, even if the underlying map is.
        
        The transformer is applied lazily, invoked when needed. This is necessary for the returned
        map to be a view, but it means that the transformer will be applied many times for bulk
        operations like Map.containsValue and Object.toString. For this to perform
        well, `transformer` should be fast. To avoid lazy evaluation when the returned map
        doesn't need to be a view, copy the returned map into a new map of your choosing.
        
        **Warning:** This method assumes that for any instance `k` of `EntryTransformer` key type `K`, `k.equals(k2)` implies that `k2` is also of
        type `K`. Using an `EntryTransformer` key type for which this may not hold, such as
        `ArrayList`, may risk a `ClassCastException` when calling methods on the
        transformed map.

        Since
        - 7.0
        """
        ...


    @staticmethod
    def transformEntries(fromMap: "SortedMap"["K", "V1"], transformer: "EntryTransformer"["K", "V1", "V2"]) -> "SortedMap"["K", "V2"]:
        """
        Returns a view of a sorted map whose values are derived from the original sorted map's entries.
        In contrast to .transformValues, this method's entry-transformation logic may depend on
        the key as well as the value.
        
        All other properties of the transformed map, such as iteration order, are left intact. For
        example, the code:
        
        ````Map<String, Boolean> options =
            ImmutableSortedMap.of("verbose", True, "sort", False);
        EntryTransformer<String, Boolean, String> flagPrefixer =
            new EntryTransformer<String, Boolean, String>() {
              public String transformEntry(String key, Boolean value) {
                return value ? key : "yes" + key;`
            };
        SortedMap<String, String> transformed =
            Maps.transformEntries(options, flagPrefixer);
        System.out.println(transformed);
        }```
        
        ... prints `{sort=yessort, verbose=verbose`}.
        
        Changes in the underlying map are reflected in this view. Conversely, this view supports
        removal operations, and these are reflected in the underlying map.
        
        It's acceptable for the underlying map to contain null keys and null values provided that
        the transformer is capable of accepting null inputs. The transformed map might contain null
        values if the transformer sometimes gives a null result.
        
        The returned map is not thread-safe or serializable, even if the underlying map is.
        
        The transformer is applied lazily, invoked when needed. This is necessary for the returned
        map to be a view, but it means that the transformer will be applied many times for bulk
        operations like Map.containsValue and Object.toString. For this to perform
        well, `transformer` should be fast. To avoid lazy evaluation when the returned map
        doesn't need to be a view, copy the returned map into a new map of your choosing.
        
        **Warning:** This method assumes that for any instance `k` of `EntryTransformer` key type `K`, `k.equals(k2)` implies that `k2` is also of
        type `K`. Using an `EntryTransformer` key type for which this may not hold, such as
        `ArrayList`, may risk a `ClassCastException` when calling methods on the
        transformed map.

        Since
        - 11.0
        """
        ...


    @staticmethod
    def transformEntries(fromMap: "NavigableMap"["K", "V1"], transformer: "EntryTransformer"["K", "V1", "V2"]) -> "NavigableMap"["K", "V2"]:
        """
        Returns a view of a navigable map whose values are derived from the original navigable map's
        entries. In contrast to .transformValues, this method's entry-transformation logic may
        depend on the key as well as the value.
        
        All other properties of the transformed map, such as iteration order, are left intact. For
        example, the code:
        
        ````NavigableMap<String, Boolean> options = Maps.newTreeMap();
        options.put("verbose", False);
        options.put("sort", True);
        EntryTransformer<String, Boolean, String> flagPrefixer =
            new EntryTransformer<String, Boolean, String>() {
              public String transformEntry(String key, Boolean value) {
                return value ? key : ("yes" + key);`
            };
        NavigableMap<String, String> transformed =
            LabsMaps.transformNavigableEntries(options, flagPrefixer);
        System.out.println(transformed);
        }```
        
        ... prints `{sort=yessort, verbose=verbose`}.
        
        Changes in the underlying map are reflected in this view. Conversely, this view supports
        removal operations, and these are reflected in the underlying map.
        
        It's acceptable for the underlying map to contain null keys and null values provided that
        the transformer is capable of accepting null inputs. The transformed map might contain null
        values if the transformer sometimes gives a null result.
        
        The returned map is not thread-safe or serializable, even if the underlying map is.
        
        The transformer is applied lazily, invoked when needed. This is necessary for the returned
        map to be a view, but it means that the transformer will be applied many times for bulk
        operations like Map.containsValue and Object.toString. For this to perform
        well, `transformer` should be fast. To avoid lazy evaluation when the returned map
        doesn't need to be a view, copy the returned map into a new map of your choosing.
        
        **Warning:** This method assumes that for any instance `k` of `EntryTransformer` key type `K`, `k.equals(k2)` implies that `k2` is also of
        type `K`. Using an `EntryTransformer` key type for which this may not hold, such as
        `ArrayList`, may risk a `ClassCastException` when calling methods on the
        transformed map.

        Since
        - 13.0
        """
        ...


    @staticmethod
    def filterKeys(unfiltered: dict["K", "V"], keyPredicate: "Predicate"["K"]) -> dict["K", "V"]:
        """
        Returns a map containing the mappings in `unfiltered` whose keys satisfy a predicate. The
        returned map is a live view of `unfiltered`; changes to one affect the other.
        
        The resulting map's `keySet()`, `entrySet()`, and `values()` views have
        iterators that don't support `remove()`, but all other methods are supported by the map
        and its views. When given a key that doesn't satisfy the predicate, the map's `put()` and
        `putAll()` methods throw an IllegalArgumentException.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered map
        or its views, only mappings whose keys satisfy the filter will be removed from the underlying
        map.
        
        The returned map isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered map's methods, such as `size()`, iterate across every key/value
        mapping in the underlying map and determine which satisfy the filter. When a live view is
        *not* needed, it may be faster to copy the filtered map and use the copy.
        
        **Warning:** `keyPredicate` must be *consistent with equals*, as documented at
        Predicate.apply. Do not provide a predicate such as `Predicates.instanceOf(ArrayList.class)`, which is inconsistent with equals.
        """
        ...


    @staticmethod
    def filterKeys(unfiltered: "SortedMap"["K", "V"], keyPredicate: "Predicate"["K"]) -> "SortedMap"["K", "V"]:
        """
        Returns a sorted map containing the mappings in `unfiltered` whose keys satisfy a
        predicate. The returned map is a live view of `unfiltered`; changes to one affect the
        other.
        
        The resulting map's `keySet()`, `entrySet()`, and `values()` views have
        iterators that don't support `remove()`, but all other methods are supported by the map
        and its views. When given a key that doesn't satisfy the predicate, the map's `put()` and
        `putAll()` methods throw an IllegalArgumentException.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered map
        or its views, only mappings whose keys satisfy the filter will be removed from the underlying
        map.
        
        The returned map isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered map's methods, such as `size()`, iterate across every key/value
        mapping in the underlying map and determine which satisfy the filter. When a live view is
        *not* needed, it may be faster to copy the filtered map and use the copy.
        
        **Warning:** `keyPredicate` must be *consistent with equals*, as documented at
        Predicate.apply. Do not provide a predicate such as `Predicates.instanceOf(ArrayList.class)`, which is inconsistent with equals.

        Since
        - 11.0
        """
        ...


    @staticmethod
    def filterKeys(unfiltered: "NavigableMap"["K", "V"], keyPredicate: "Predicate"["K"]) -> "NavigableMap"["K", "V"]:
        """
        Returns a navigable map containing the mappings in `unfiltered` whose keys satisfy a
        predicate. The returned map is a live view of `unfiltered`; changes to one affect the
        other.
        
        The resulting map's `keySet()`, `entrySet()`, and `values()` views have
        iterators that don't support `remove()`, but all other methods are supported by the map
        and its views. When given a key that doesn't satisfy the predicate, the map's `put()` and
        `putAll()` methods throw an IllegalArgumentException.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered map
        or its views, only mappings whose keys satisfy the filter will be removed from the underlying
        map.
        
        The returned map isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered map's methods, such as `size()`, iterate across every key/value
        mapping in the underlying map and determine which satisfy the filter. When a live view is
        *not* needed, it may be faster to copy the filtered map and use the copy.
        
        **Warning:** `keyPredicate` must be *consistent with equals*, as documented at
        Predicate.apply. Do not provide a predicate such as `Predicates.instanceOf(ArrayList.class)`, which is inconsistent with equals.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def filterKeys(unfiltered: "BiMap"["K", "V"], keyPredicate: "Predicate"["K"]) -> "BiMap"["K", "V"]:
        """
        Returns a bimap containing the mappings in `unfiltered` whose keys satisfy a predicate.
        The returned bimap is a live view of `unfiltered`; changes to one affect the other.
        
        The resulting bimap's `keySet()`, `entrySet()`, and `values()` views have
        iterators that don't support `remove()`, but all other methods are supported by the bimap
        and its views. When given a key that doesn't satisfy the predicate, the bimap's `put()`,
        `forcePut()` and `putAll()` methods throw an IllegalArgumentException.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered
        bimap or its views, only mappings that satisfy the filter will be removed from the underlying
        bimap.
        
        The returned bimap isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered bimap's methods, such as `size()`, iterate across every key in
        the underlying bimap and determine which satisfy the filter. When a live view is *not*
        needed, it may be faster to copy the filtered bimap and use the copy.
        
        **Warning:** `entryPredicate` must be *consistent with equals *, as documented
        at Predicate.apply.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def filterValues(unfiltered: dict["K", "V"], valuePredicate: "Predicate"["V"]) -> dict["K", "V"]:
        """
        Returns a map containing the mappings in `unfiltered` whose values satisfy a predicate.
        The returned map is a live view of `unfiltered`; changes to one affect the other.
        
        The resulting map's `keySet()`, `entrySet()`, and `values()` views have
        iterators that don't support `remove()`, but all other methods are supported by the map
        and its views. When given a value that doesn't satisfy the predicate, the map's `put()`,
        `putAll()`, and Entry.setValue methods throw an IllegalArgumentException.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered map
        or its views, only mappings whose values satisfy the filter will be removed from the underlying
        map.
        
        The returned map isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered map's methods, such as `size()`, iterate across every key/value
        mapping in the underlying map and determine which satisfy the filter. When a live view is
        *not* needed, it may be faster to copy the filtered map and use the copy.
        
        **Warning:** `valuePredicate` must be *consistent with equals*, as documented
        at Predicate.apply. Do not provide a predicate such as `Predicates.instanceOf(ArrayList.class)`, which is inconsistent with equals.
        """
        ...


    @staticmethod
    def filterValues(unfiltered: "SortedMap"["K", "V"], valuePredicate: "Predicate"["V"]) -> "SortedMap"["K", "V"]:
        """
        Returns a sorted map containing the mappings in `unfiltered` whose values satisfy a
        predicate. The returned map is a live view of `unfiltered`; changes to one affect the
        other.
        
        The resulting map's `keySet()`, `entrySet()`, and `values()` views have
        iterators that don't support `remove()`, but all other methods are supported by the map
        and its views. When given a value that doesn't satisfy the predicate, the map's `put()`,
        `putAll()`, and Entry.setValue methods throw an IllegalArgumentException.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered map
        or its views, only mappings whose values satisfy the filter will be removed from the underlying
        map.
        
        The returned map isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered map's methods, such as `size()`, iterate across every key/value
        mapping in the underlying map and determine which satisfy the filter. When a live view is
        *not* needed, it may be faster to copy the filtered map and use the copy.
        
        **Warning:** `valuePredicate` must be *consistent with equals*, as documented
        at Predicate.apply. Do not provide a predicate such as `Predicates.instanceOf(ArrayList.class)`, which is inconsistent with equals.

        Since
        - 11.0
        """
        ...


    @staticmethod
    def filterValues(unfiltered: "NavigableMap"["K", "V"], valuePredicate: "Predicate"["V"]) -> "NavigableMap"["K", "V"]:
        """
        Returns a navigable map containing the mappings in `unfiltered` whose values satisfy a
        predicate. The returned map is a live view of `unfiltered`; changes to one affect the
        other.
        
        The resulting map's `keySet()`, `entrySet()`, and `values()` views have
        iterators that don't support `remove()`, but all other methods are supported by the map
        and its views. When given a value that doesn't satisfy the predicate, the map's `put()`,
        `putAll()`, and Entry.setValue methods throw an IllegalArgumentException.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered map
        or its views, only mappings whose values satisfy the filter will be removed from the underlying
        map.
        
        The returned map isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered map's methods, such as `size()`, iterate across every key/value
        mapping in the underlying map and determine which satisfy the filter. When a live view is
        *not* needed, it may be faster to copy the filtered map and use the copy.
        
        **Warning:** `valuePredicate` must be *consistent with equals*, as documented
        at Predicate.apply. Do not provide a predicate such as `Predicates.instanceOf(ArrayList.class)`, which is inconsistent with equals.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def filterValues(unfiltered: "BiMap"["K", "V"], valuePredicate: "Predicate"["V"]) -> "BiMap"["K", "V"]:
        """
        Returns a bimap containing the mappings in `unfiltered` whose values satisfy a predicate.
        The returned bimap is a live view of `unfiltered`; changes to one affect the other.
        
        The resulting bimap's `keySet()`, `entrySet()`, and `values()` views have
        iterators that don't support `remove()`, but all other methods are supported by the bimap
        and its views. When given a value that doesn't satisfy the predicate, the bimap's `put()`, `forcePut()` and `putAll()` methods throw an IllegalArgumentException. Similarly, the map's entries have a Entry.setValue method
        that throws an IllegalArgumentException when the provided value doesn't satisfy the
        predicate.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered
        bimap or its views, only mappings that satisfy the filter will be removed from the underlying
        bimap.
        
        The returned bimap isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered bimap's methods, such as `size()`, iterate across every value in
        the underlying bimap and determine which satisfy the filter. When a live view is *not*
        needed, it may be faster to copy the filtered bimap and use the copy.
        
        **Warning:** `entryPredicate` must be *consistent with equals *, as documented
        at Predicate.apply.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def filterEntries(unfiltered: dict["K", "V"], entryPredicate: "Predicate"["Entry"["K", "V"]]) -> dict["K", "V"]:
        """
        Returns a map containing the mappings in `unfiltered` that satisfy a predicate. The
        returned map is a live view of `unfiltered`; changes to one affect the other.
        
        The resulting map's `keySet()`, `entrySet()`, and `values()` views have
        iterators that don't support `remove()`, but all other methods are supported by the map
        and its views. When given a key/value pair that doesn't satisfy the predicate, the map's `put()` and `putAll()` methods throw an IllegalArgumentException. Similarly, the
        map's entries have a Entry.setValue method that throws an IllegalArgumentException when the existing key and the provided value don't satisfy the
        predicate.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered map
        or its views, only mappings that satisfy the filter will be removed from the underlying map.
        
        The returned map isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered map's methods, such as `size()`, iterate across every key/value
        mapping in the underlying map and determine which satisfy the filter. When a live view is
        *not* needed, it may be faster to copy the filtered map and use the copy.
        
        **Warning:** `entryPredicate` must be *consistent with equals*, as documented
        at Predicate.apply.
        """
        ...


    @staticmethod
    def filterEntries(unfiltered: "SortedMap"["K", "V"], entryPredicate: "Predicate"["Entry"["K", "V"]]) -> "SortedMap"["K", "V"]:
        """
        Returns a sorted map containing the mappings in `unfiltered` that satisfy a predicate.
        The returned map is a live view of `unfiltered`; changes to one affect the other.
        
        The resulting map's `keySet()`, `entrySet()`, and `values()` views have
        iterators that don't support `remove()`, but all other methods are supported by the map
        and its views. When given a key/value pair that doesn't satisfy the predicate, the map's `put()` and `putAll()` methods throw an IllegalArgumentException. Similarly, the
        map's entries have a Entry.setValue method that throws an IllegalArgumentException when the existing key and the provided value don't satisfy the
        predicate.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered map
        or its views, only mappings that satisfy the filter will be removed from the underlying map.
        
        The returned map isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered map's methods, such as `size()`, iterate across every key/value
        mapping in the underlying map and determine which satisfy the filter. When a live view is
        *not* needed, it may be faster to copy the filtered map and use the copy.
        
        **Warning:** `entryPredicate` must be *consistent with equals*, as documented
        at Predicate.apply.

        Since
        - 11.0
        """
        ...


    @staticmethod
    def filterEntries(unfiltered: "NavigableMap"["K", "V"], entryPredicate: "Predicate"["Entry"["K", "V"]]) -> "NavigableMap"["K", "V"]:
        """
        Returns a sorted map containing the mappings in `unfiltered` that satisfy a predicate.
        The returned map is a live view of `unfiltered`; changes to one affect the other.
        
        The resulting map's `keySet()`, `entrySet()`, and `values()` views have
        iterators that don't support `remove()`, but all other methods are supported by the map
        and its views. When given a key/value pair that doesn't satisfy the predicate, the map's `put()` and `putAll()` methods throw an IllegalArgumentException. Similarly, the
        map's entries have a Entry.setValue method that throws an IllegalArgumentException when the existing key and the provided value don't satisfy the
        predicate.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered map
        or its views, only mappings that satisfy the filter will be removed from the underlying map.
        
        The returned map isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered map's methods, such as `size()`, iterate across every key/value
        mapping in the underlying map and determine which satisfy the filter. When a live view is
        *not* needed, it may be faster to copy the filtered map and use the copy.
        
        **Warning:** `entryPredicate` must be *consistent with equals*, as documented
        at Predicate.apply.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def filterEntries(unfiltered: "BiMap"["K", "V"], entryPredicate: "Predicate"["Entry"["K", "V"]]) -> "BiMap"["K", "V"]:
        """
        Returns a bimap containing the mappings in `unfiltered` that satisfy a predicate. The
        returned bimap is a live view of `unfiltered`; changes to one affect the other.
        
        The resulting bimap's `keySet()`, `entrySet()`, and `values()` views have
        iterators that don't support `remove()`, but all other methods are supported by the bimap
        and its views. When given a key/value pair that doesn't satisfy the predicate, the bimap's
        `put()`, `forcePut()` and `putAll()` methods throw an IllegalArgumentException. Similarly, the map's entries have an Entry.setValue method
        that throws an IllegalArgumentException when the existing key and the provided value
        don't satisfy the predicate.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered
        bimap or its views, only mappings that satisfy the filter will be removed from the underlying
        bimap.
        
        The returned bimap isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered bimap's methods, such as `size()`, iterate across every key/value
        mapping in the underlying bimap and determine which satisfy the filter. When a live view is
        *not* needed, it may be faster to copy the filtered bimap and use the copy.
        
        **Warning:** `entryPredicate` must be *consistent with equals *, as documented
        at Predicate.apply.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def unmodifiableNavigableMap(map: "NavigableMap"["K", "V"]) -> "NavigableMap"["K", "V"]:
        """
        Returns an unmodifiable view of the specified navigable map. Query operations on the returned
        map read through to the specified map, and attempts to modify the returned map, whether direct
        or via its views, result in an `UnsupportedOperationException`.
        
        The returned navigable map will be serializable if the specified navigable map is
        serializable.
        
        This method's signature will not permit you to convert a `NavigableMap<? extends K,
        V>` to a `NavigableMap<K, V>`. If it permitted this, the returned map's `comparator()` method might return a `Comparator<? extends K>`, which works only on a
        particular subtype of `K`, but promise that it's a `Comparator<? super K>`, which
        must work on any type of `K`.

        Arguments
        - map: the navigable map for which an unmodifiable view is to be returned

        Returns
        - an unmodifiable view of the specified navigable map

        Since
        - 12.0
        """
        ...


    @staticmethod
    def synchronizedNavigableMap(navigableMap: "NavigableMap"["K", "V"]) -> "NavigableMap"["K", "V"]:
        """
        Returns a synchronized (thread-safe) navigable map backed by the specified navigable map. In
        order to guarantee serial access, it is critical that **all** access to the backing
        navigable map is accomplished through the returned navigable map (or its views).
        
        It is imperative that the user manually synchronize on the returned navigable map when
        iterating over any of its collection views, or the collections views of any of its `descendingMap`, `subMap`, `headMap` or `tailMap` views.
        
        ````NavigableMap<K, V> map = synchronizedNavigableMap(new TreeMap<K, V>());
        
        // Needn't be in synchronized block
        NavigableSet<K> set = map.navigableKeySet();
        
        synchronized (map) { // Synchronizing on map, not set!
          Iterator<K> it = set.iterator(); // Must be in synchronized block
          while (it.hasNext()) {
            foo(it.next());`
        }
        }```
        
        or:
        
        ````NavigableMap<K, V> map = synchronizedNavigableMap(new TreeMap<K, V>());
        NavigableMap<K, V> map2 = map.subMap(foo, False, bar, True);
        
        // Needn't be in synchronized block
        NavigableSet<K> set2 = map2.descendingKeySet();
        
        synchronized (map) { // Synchronizing on map, not map2 or set2!
          Iterator<K> it = set2.iterator(); // Must be in synchronized block
          while (it.hasNext()) {
            foo(it.next());`
        }
        }```
        
        Failure to follow this advice may result in non-deterministic behavior.
        
        The returned navigable map will be serializable if the specified navigable map is
        serializable.

        Arguments
        - navigableMap: the navigable map to be "wrapped" in a synchronized navigable map.

        Returns
        - a synchronized view of the specified navigable map.

        Since
        - 13.0
        """
        ...


    @staticmethod
    def subMap(map: "NavigableMap"["K", "V"], range: "Range"["K"]) -> "NavigableMap"["K", "V"]:
        """
        Returns a view of the portion of `map` whose keys are contained by `range`.
        
        This method delegates to the appropriate methods of NavigableMap (namely NavigableMap.subMap(Object, boolean, Object, boolean) subMap(), NavigableMap.tailMap(Object, boolean) tailMap(), and NavigableMap.headMap(Object,
        boolean) headMap()) to actually construct the view. Consult these methods for a full
        description of the returned view's behavior.
        
        **Warning:** `Range`s always represent a range of values using the values' natural
        ordering. `NavigableMap` on the other hand can specify a custom ordering via a Comparator, which can violate the natural ordering. Using this method (or in general using
        `Range`) with unnaturally-ordered maps can lead to unexpected and undefined behavior.

        Since
        - 20.0
        """
        ...


    class EntryTransformer:
        """
        A transformation of the value of a key-value pair, using both key and value as inputs. To apply
        the transformation to a map, use Maps.transformEntries(Map, EntryTransformer).
        
        Type `<K>`: the key type of the input and output entries

        Arguments
        - <V1>: the value type of the input entry
        - <V2>: the value type of the output entry

        Since
        - 7.0
        """

        def transformEntry(self, key: "K", value: "V1") -> "V2":
            """
            Determines an output value based on a key-value pair. This method is *generally
            expected*, but not absolutely required, to have the following properties:
            
            
              - Its execution does not cause any observable side effects.
              - The computation is *consistent with equals*; that is, Objects.equal
                  Objects.equal`(k1, k2) &&` Objects.equal`(v1, v2)` implies that
                  `Objects.equal(transformer.transform(k1, v1), transformer.transform(k2, v2))`.

            Raises
            - NullPointerException: if the key or value is null and this transformer does not accept
                null arguments
            """
            ...



"""
Python module generated from Java source file com.google.gson.JsonParser

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.errorprone.annotations import InlineMe
from com.google.gson import *
from com.google.gson.internal import Streams
from com.google.gson.stream import JsonReader
from com.google.gson.stream import JsonToken
from com.google.gson.stream import MalformedJsonException
from java.io import IOException
from java.io import Reader
from java.io import StringReader
from typing import Any, Callable, Iterable, Tuple


class JsonParser:
    """
    A parser to parse JSON into a parse tree of JsonElements.
    
    The JSON data is parsed in JsonReader.setStrictness(Strictness) lenient mode.
    
    Here's an example of parsing from a string:
    
    ```
    String json = "{\"key\": \"value\"}";
    JsonElement jsonElement = JsonParser.parseString(json);
    JsonObject jsonObject = jsonElement.getAsJsonObject();
    ```
    
    It can also parse from a reader:
    
    ```
    try (Reader reader = new FileReader("my-data.json", StandardCharsets.UTF_8)) {
      JsonElement jsonElement = JsonParser.parseReader(reader);
      JsonObject jsonObject = jsonElement.getAsJsonObject();
    }
    ```
    
    If you want to parse from a JsonReader for more customized parsing requirements, the
    following example demonstrates how to achieve it:
    
    ```
    String json = "{\"skipObj\": {\"skipKey\": \"skipValue\"}, \"obj\": {\"key\": \"value\"}}";
    try (JsonReader jsonReader = new JsonReader(new StringReader(json))) {
      jsonReader.beginObject();
      while (jsonReader.hasNext()) {
        String fieldName = jsonReader.nextName();
        if (fieldName.equals("skipObj")) {
          jsonReader.skipValue();
        } else {
          JsonElement jsonElement = JsonParser.parseReader(jsonReader);
          JsonObject jsonObject = jsonElement.getAsJsonObject();
        }
      }
      jsonReader.endObject();
    }
    ```

    Author(s)
    - Joel Leitch

    Since
    - 1.3
    """

    def __init__(self):
        """
        Deprecated
        - No need to instantiate this class, use the static methods instead.
        """
        ...


    @staticmethod
    def parseString(json: str) -> "JsonElement":
        """
        Parses the specified JSON string into a parse tree. An exception is thrown if the JSON string
        has multiple top-level JSON elements, or if there is trailing data.
        
        The JSON string is parsed in JsonReader.setStrictness(Strictness) lenient mode.

        Arguments
        - json: JSON text

        Returns
        - a parse tree of JsonElements corresponding to the specified JSON

        Raises
        - JsonParseException: if the specified text is not valid JSON

        Since
        - 2.8.6
        """
        ...


    @staticmethod
    def parseReader(reader: "Reader") -> "JsonElement":
        """
        Parses the complete JSON string provided by the reader into a parse tree. An exception is
        thrown if the JSON string has multiple top-level JSON elements, or if there is trailing data.
        
        The JSON data is parsed in JsonReader.setStrictness(Strictness) lenient mode.

        Arguments
        - reader: JSON text

        Returns
        - a parse tree of JsonElements corresponding to the specified JSON

        Raises
        - JsonParseException: if there is an IOException or if the specified text is not valid
            JSON

        Since
        - 2.8.6
        """
        ...


    @staticmethod
    def parseReader(reader: "JsonReader") -> "JsonElement":
        """
        Returns the next value from the JSON stream as a parse tree. Unlike the other `parse`
        methods, no exception is thrown if the JSON data has multiple top-level JSON elements, or if
        there is trailing data.
        
        If the JsonReader.getStrictness() strictness of the reader is Strictness.STRICT, that strictness will be used for parsing. Otherwise the strictness will be
        temporarily changed to Strictness.LENIENT and will be restored once this method
        returns.

        Raises
        - JsonParseException: if there is an IOException or if the specified text is not valid
            JSON

        Since
        - 2.8.6
        """
        ...


    def parse(self, json: str) -> "JsonElement":
        """
        Deprecated
        - Use JsonParser.parseString
        """
        ...


    def parse(self, json: "Reader") -> "JsonElement":
        """
        Deprecated
        - Use JsonParser.parseReader(Reader)
        """
        ...


    def parse(self, json: "JsonReader") -> "JsonElement":
        """
        Deprecated
        - Use JsonParser.parseReader(JsonReader)
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.LinkedListMultimap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.j2objc.annotations import WeakOuter
from java.io import IOException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from java.io import Serializable
from java.util import AbstractSequentialList
from java.util import ConcurrentModificationException
from java.util import Iterator
from java.util import ListIterator
from java.util import NoSuchElementException
from java.util.function import Consumer
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class LinkedListMultimap(AbstractMultimap, ListMultimap, Serializable):
    """
    An implementation of `ListMultimap` that supports deterministic iteration order for both
    keys and values. The iteration order is preserved across non-distinct key values. For example,
    for the following multimap definition:
    
    ````Multimap<K, V> multimap = LinkedListMultimap.create();
    multimap.put(key1, foo);
    multimap.put(key2, bar);
    multimap.put(key1, baz);````
    
    ... the iteration order for .keys() is `[key1, key2, key1]`, and similarly for
    .entries(). Unlike LinkedHashMultimap, the iteration order is kept consistent
    between keys, entries and values. For example, calling:
    
    ````multimap.remove(key1, foo);````
    
    changes the entries iteration order to `[key2=bar, key1=baz]` and the key iteration
    order to `[key2, key1]`. The .entries() iterator returns mutable map entries, and
    .replaceValues attempts to preserve iteration order as much as possible.
    
    The collections returned by .keySet() and .asMap iterate through the keys in
    the order they were first added to the multimap. Similarly, .get, .removeAll, and
    .replaceValues return collections that iterate through the values in the order they were
    added. The collections generated by .entries(), .keys(), and .values
    iterate across the key-value mappings in the order they were added to the multimap.
    
    The .values() and .entries() methods both return a `List`, instead of
    the `Collection` specified by the ListMultimap interface.
    
    The methods .get, .keySet(), .keys(), .values, .entries(), and .asMap return collections that are views of the multimap. If the
    multimap is modified while an iteration over any of those collections is in progress, except
    through the iterator's methods, the results of the iteration are undefined.
    
    Keys and values may be null. All optional multimap methods are supported, and all returned
    views are modifiable.
    
    This class is not threadsafe when any concurrent operations update the multimap. Concurrent
    read operations will work correctly. To allow concurrent update operations, wrap your multimap
    with a call to Multimaps.synchronizedListMultimap.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/NewCollectionTypesExplained#multimap">`Multimap`</a>.

    Author(s)
    - Mike Bostock

    Since
    - 2.0
    """

    @staticmethod
    def create() -> "LinkedListMultimap"["K", "V"]:
        """
        Creates a new, empty `LinkedListMultimap` with the default initial capacity.
        """
        ...


    @staticmethod
    def create(expectedKeys: int) -> "LinkedListMultimap"["K", "V"]:
        """
        Constructs an empty `LinkedListMultimap` with enough capacity to hold the specified
        number of keys without rehashing.

        Arguments
        - expectedKeys: the expected number of distinct keys

        Raises
        - IllegalArgumentException: if `expectedKeys` is negative
        """
        ...


    @staticmethod
    def create(multimap: "Multimap"["K", "V"]) -> "LinkedListMultimap"["K", "V"]:
        """
        Constructs a `LinkedListMultimap` with the same mappings as the specified `Multimap`. The new multimap has the same Multimap.entries() iteration order as the
        input multimap.

        Arguments
        - multimap: the multimap whose contents are copied to this multimap
        """
        ...


    def size(self) -> int:
        ...


    def isEmpty(self) -> bool:
        ...


    def containsKey(self, key: "Object") -> bool:
        ...


    def containsValue(self, value: "Object") -> bool:
        ...


    def put(self, key: "K", value: "V") -> bool:
        """
        Stores a key-value pair in the multimap.

        Arguments
        - key: key to store in the multimap
        - value: value to store in the multimap

        Returns
        - `True` always
        """
        ...


    def replaceValues(self, key: "K", values: Iterable["V"]) -> list["V"]:
        """
        
        
        If any entries for the specified `key` already exist in the multimap, their values are
        changed in-place without affecting the iteration order.
        
        The returned list is immutable and implements java.util.RandomAccess.
        """
        ...


    def removeAll(self, key: "Object") -> list["V"]:
        """
        
        
        The returned list is immutable and implements java.util.RandomAccess.
        """
        ...


    def clear(self) -> None:
        ...


    def get(self, key: "K") -> list["V"]:
        """
        
        
        If the multimap is modified while an iteration over the list is in progress (except through
        the iterator's own `add`, `set` or `remove` operations) the results of the
        iteration are undefined.
        
        The returned list is not serializable and does not have random access.
        """
        ...


    def values(self) -> list["V"]:
        """
        
        
        The iterator generated by the returned collection traverses the values in the order they
        were added to the multimap. Because the values may have duplicates and follow the insertion
        ordering, this method returns a List, instead of the Collection specified in
        the ListMultimap interface.
        """
        ...


    def entries(self) -> list["Entry"["K", "V"]]:
        """
        
        
        The iterator generated by the returned collection traverses the entries in the order they
        were added to the multimap. Because the entries may have duplicates and follow the insertion
        ordering, this method returns a List, instead of the Collection specified in
        the ListMultimap interface.
        
        An entry's Entry.getKey method always returns the same key, regardless of what
        happens subsequently. As long as the corresponding key-value mapping is not removed from the
        multimap, Entry.getValue returns the value from the multimap, which may change over
        time, and Entry.setValue modifies that value. Removing the mapping from the multimap
        does not alter the value returned by `getValue()`, though a subsequent `setValue()`
        call won't update the multimap but will lead to a revised value being returned by `getValue()`.
        """
        ...
